package com.qianwankj.vt.vonage.flutter_vonage

import android.content.Context
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.EventChannel.EventSink
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

class FlutterVonagePlugin : FlutterPlugin, MethodCallHandler {

    lateinit var methodChannel: MethodChannel
    lateinit var eventChannel: EventChannel
    private lateinit var mVonageUtils: VonageUtils
    private lateinit var mContext: Context

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        mContext = flutterPluginBinding.applicationContext
        mVonageUtils = VonageUtils(mContext)

        val messenger = flutterPluginBinding.binaryMessenger
        methodChannel = MethodChannel(messenger, Constants.METHOD_CHANNEL_NAME)
        eventChannel = EventChannel(messenger, Constants.EVENT_CHANNEL_NAME)

        methodChannel.setMethodCallHandler(this)
        eventChannel.setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(arguments: Any?, events: EventSink) {
                mVonageUtils.eventSink = events
            }

            override fun onCancel(arguments: Any?) {
                mVonageUtils.eventSink = null
            }
        })
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        mVonageUtils.destroy()
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        mVonageUtils.onMethodCall(mContext, call, result)
    }
}
