import 'dart:convert' as convert;

import 'package:flutter/material.dart';
import 'package:flutter_vonage/flutter_vonage.dart';

/// 电话界面
class VoicePage extends StatefulWidget {
  final String? curName;
  final String? targetName;

  VoicePage(this.curName, this.targetName);

  @override
  State<StatefulWidget> createState() {
    return _VoicePageState();
  }
}

class _VoicePageState extends State<VoicePage> {
  List<String> mList = [];
  late VonageController _controller;
  int status = 0; // 0 正常状态, 1 拨号状态 ，2 来电状态
  String statusDesc = "";

  @override
  void initState() {
    super.initState();
    _controller = VonageController();
    _controller.addVonageCallback("voice", (type, info) {
      if (type == Constants.INCOMING_CALL) {
        var resp = convert.jsonDecode(info);
        var desc = "${resp['sender']['name']} 来电话了";
        setState(() {
          status = 2;
          statusDesc = desc;
        });
        return;
      }

      if (type == Constants.INCOMING_CALL_ANSWER) {
        var desc = "电话已接通 $info";
        setState(() {
          status = 1;
          statusDesc = desc;
        });
        return;
      }

      if (type == Constants.INCOMING_CALL_REJECT) {
        var desc = "电话已拒绝 $info";
        setState(() {
          status = 0;
          statusDesc = desc;
        });
        return;
      }

      if (type == Constants.OUTGOING_CALL) {
        var desc = "拨号完毕 $info";
        setState(() {
          status = 1;
          statusDesc = desc;
        });
        return;
      }

      if (type == Constants.CLOSE_CALL) {
        var desc = "电话挂断 $info";
        setState(() {
          status = 0;
          statusDesc = desc;
        });
        return;
      }
    });
  }

  @override
  void dispose() {
    super.dispose();
    _controller.removeVonageCallback("voice");
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(title: Text("通话演示")),
      body: Column(
        children: [
          Text(statusDesc),
          Expanded(
            child: Center(
              child: Row(
                children: [
                  status == 0
                      ? TextButton(onPressed: () => dial(), child: Text("拨号"))
                      : Container(),
                  status == 1
                      ? TextButton(onPressed: () => callEnd(), child: Text("取消"))
                      : Container(),
                  status == 2
                      ? TextButton(onPressed: () => callAnswer(), child: Text("接通"))
                      : Container(),
                  status == 2
                      ? TextButton(onPressed: () => callReject(), child: Text("拒绝"))
                      : Container(),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }

  dial() async {
    var fromNumber = widget.curName ?? "";
    var toNumber = widget.targetName ?? "";
    var type = "IN_APP"; // IN_APP , SERVER
    var params = DialParams(type, fromNumber, toNumber);
    await _controller.dial(params);
  }

  callEnd() async {
    await _controller.callEnd();
  }

  callAnswer() async {
    await _controller.callAnswer();
  }

  callReject() async {
    await _controller.callReject();
  }
}
