import 'dart:async';
import 'dart:convert' as convert;

import 'package:flutter/services.dart';

import 'constants.dart';
import 'vonage_params.dart';

/// 回调
typedef VonageCallback = void Function(String type, String info);

/// 控制器
class VonageController {
  late MethodChannel _channel;
  final Map<String, VonageCallback> callbackList = {};

  VonageController() {
    _channel = MethodChannel(Constants.METHOD_CHANNEL_NAME);

    /// 监听原生事件回调
    EventChannel(Constants.EVENT_CHANNEL_NAME)
        .receiveBroadcastStream()
        .listen(_eventData, onError: _eventError, onDone: _eventDone);
  }

  /// 出错回调
  void _eventError(error) {
    print("@@@@@eventError:$error");
  }

  /// 接收数据回调
  void _eventData(event) {
    print("@@@@@eventData:$event");
    var resp = convert.jsonDecode(event);

    callbackList.forEach((key, value) {
      value.call(resp["type"], resp["info"]);
    });
  }

  /// 关闭并发送完成事件流，回调
  void _eventDone() {
    print("@@@@@eventDone");
  }

  addVonageCallback(String key, VonageCallback callback) {
    callbackList[key] = callback;
  }

  removeVonageCallback(String key) {
    callbackList.remove(key);
  }

  Future init(InitParams params) async {
    return await _channel.invokeMethod(Constants.CMD_INIT, params.toJson());
  }

  Future close() async {
    return await _channel.invokeMethod(Constants.CMD_CLOSE);
  }

  Future login(LoginParams params) async {
    return await _channel.invokeMethod(Constants.CMD_LOGIN, params.toJson());
  }

  Future logout() async {
    return await _channel.invokeMethod(Constants.CMD_LOGOUT);
  }

  Future updatePushToken(PushTokenParams params) async {
    return await _channel.invokeMethod(Constants.CMD_UPDATE_PUSH_TOKEN, params.toJson());
  }

  Future handlePush(Map<String, dynamic> params) async {
    return await _channel.invokeMethod(Constants.CMD_HANDEL_PUSH, params);
  }

  Future startChat(ChatParams params) async {
    return await _channel.invokeMethod(Constants.CMD_START_CHAT, params.toJson());
  }

  Future sendMessage(MessageParams params) async {
    return await _channel.invokeMethod(Constants.CMD_SEND_MESSAGE, params.toJson());
  }

  Future dial(DialParams params) async {
    return await _channel.invokeMethod(Constants.CMD_CALL_DIAL, params.toJson());
  }

  Future callEnd() async {
    return await _channel.invokeMethod(Constants.CMD_CALL_END);
  }

  Future callAnswer() async {
    return await _channel.invokeMethod(Constants.CMD_CALL_ANSWER);
  }

  Future callReject() async {
    return await _channel.invokeMethod(Constants.CMD_CALL_REJECT);
  }

  Future mute(MuteParams params) async {
    return await _channel.invokeMethod(Constants.CMD_CALL_EVENT_MUTE, params.toJson());
  }

  Future speak(SpeakParams params) async {
    return await _channel.invokeMethod(Constants.CMD_CALL_EVENT_SPEAK, params.toJson());
  }

  Future record(RecordParams params) async {
    return await _channel.invokeMethod(Constants.CMD_CALL_EVENT_RECORD, params.toJson());
  }

  Future dtmf(DtmfParams params) async {
    return await _channel.invokeMethod(Constants.CMD_CALL_EVENT_DTMF, params.toJson());
  }
}
